import { Suspense } from "react"
import ArticleList from "@/components/article-list"
import { TagFilter } from "@/components/tag-filter"
import { Skeleton } from "@/components/ui/skeleton"

export default function ArticlesPage() {
  return (
    <main className="container mx-auto py-8 px-4">
      <h1 className="text-3xl font-bold mb-8">Articles</h1>

      <div className="grid grid-cols-1 lg:grid-cols-4 gap-8">
        <div className="lg:col-span-1">
          <div className="sticky top-8">
            <h2 className="text-xl font-semibold mb-4">Filter by Tags</h2>
            <Suspense fallback={<Skeleton className="h-64 w-full" />}>
              <TagFilter />
            </Suspense>
          </div>
        </div>

        <div className="lg:col-span-3">
          <Suspense fallback={<ArticleListSkeleton />}>
            <ArticleList />
          </Suspense>
        </div>
      </div>
    </main>
  )
}

function ArticleListSkeleton() {
  return (
    <div className="grid grid-cols-1 md:grid-cols-2 gap-6">
      {Array.from({ length: 6 }).map((_, i) => (
        <div key={i} className="flex flex-col gap-2">
          <Skeleton className="h-48 w-full rounded-lg" />
          <Skeleton className="h-6 w-3/4" />
          <Skeleton className="h-4 w-full" />
          <Skeleton className="h-4 w-full" />
          <Skeleton className="h-4 w-2/3" />
        </div>
      ))}
    </div>
  )
} 