import Image from "next/image"
import Link from "next/link"
import { formatDate } from "@/lib/utils"
import type { Article } from "@/lib/types"
import { Badge } from "@/components/ui/badge"
import { Card, CardContent, CardFooter, CardHeader } from "@/components/ui/card"

interface ArticleCardProps {
  article: Article
}

export default function ArticleCard({ article }: ArticleCardProps) {
  return (
    <Card className="h-full flex flex-col overflow-hidden hover:shadow-md transition-shadow">
      <div className="relative h-48 w-full">
        {article.image ? (
          <Image src={article.image || "/placeholder.svg"} alt={article.title} fill className="object-cover" />
        ) : (
          <div className="w-full h-full bg-muted flex items-center justify-center">
            <span className="text-muted-foreground">No image</span>
          </div>
        )}
      </div>

      <CardHeader className="pb-2">
        <Link href={`/articles/${article.id}`} className="hover:underline">
          <h3 className="text-xl font-semibold line-clamp-2">{article.title}</h3>
        </Link>
        <p className="text-sm text-muted-foreground">{formatDate(article.created)}</p>
      </CardHeader>

      <CardContent className="flex-grow">
        <p className="line-clamp-3 text-muted-foreground">{article.summary}</p>
      </CardContent>

      <CardFooter className="flex flex-wrap gap-2">
        {article.tags.map((tag) => (
          <Link key={tag.id} href={`/?tag=${tag.id}`}>
            <Badge variant="secondary" className="hover:bg-secondary/80">
              {tag.name}
            </Badge>
          </Link>
        ))}
      </CardFooter>
    </Card>
  )
}

